///////////////////////////////////////////////
//                                           //
// VRLab Mental Vision Module 0: Mix Parabol //
// Author: Achille Peternier                 //
// Copyright: EPFL-VRLab 2005 (C)            //
//                                           //
///////////////////////////////////////////////
#include "../data/common.h"
#pragma comment(lib, "opengl32.lib")



//////////////
// #DEFINES //
//////////////
#define MODULE_NAME "Mixing parabola"



//////////////
// ACCURACY //
//////////////
#define ACCURACY 2000  // Lines per fragment
#define POINTS   4     // Control points



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool parametricCurve(float t, MVVECTOR2 p0, MVVECTOR2 p1, MVVECTOR2 p2, MVVECTOR2 *a0, MVVECTOR2 *a1, MVVECTOR2 *a2) //
{
   if (t==1.0f) t= 0.999999999f;
   *a0 = p0;   
   *a2 = ((p1-p0) - (p2-p0)*t)/t*(1.0f-t);
   *a1 = p2 - p0 - *a2;
   // Done:
   return true;
}


//////////////////////////////////////////////////////////////////////////////////////////////////
bool parametricCurveBis(MVVECTOR2 p0, MVVECTOR2 p1, MVVECTOR2 p2, float *a, float *b, float *c) //
{   
   *a = ( p0.x*(p1.y-p2.y) + p0.y*(p2.x-p1.x) + p1.x*p2.y - p2.x*p1.y )/
        ( (p1.x-p0.x)*(p2.x-p0.x)*(p2.x-p1.x) );
        
   *b = (p2.y-p0.y)/(p2.x-p0.x) - (p2.x+p0.x)* (*a);
   
   *c = p0.y - (p0.x * p0.x)* (*a) - (*b)*p0.x;
   
   // Done:
   return true;
}

MVVECTOR2 mixedParable(float t, float a0, float b0, float c0, float a1, float b1, float c1, MVVECTOR2 p1, MVVECTOR2 p2)
{
   MVVECTOR2 p, _p1, _p2;
   float norm = p2.distance(p1);
   _p1.x = t;
   _p2.x = t;
   _p1.y = pow(_p1.x, 2.0f)*a0 + b0*_p1.x + c0;
   _p2.y = pow(_p2.x, 2.0f)*a1 + b1*_p1.x + c1;
   p = _p1 * (1.0f-t/norm) + (_p2/norm) * t;
   return p;
}



//////////
// MAIN //
//////////

int main(int argc, char *argv[])
{ 
   /////////////////
   // Init MVisio //
   /////////////////
   MVOPENGL::setNearPlane(0.01f);   
   MVOPENGL::setFarPlane(100.0f);
   INIT_MVISIO(MODULE_NAME, argc, argv);    
   
   // Common interface:
   USE_FONT();      
   USE_BACKGROUND(MODULE_SIZEX, MODULE_SIZEY);
   USE_QUITBUTTON(MODULE_SIZEX, MODULE_SIZEY);
  
   
   ////////////
   // Curves //
   ////////////     
   
   // Params:
   float param[POINTS*3];
   
   // Pulse:
   float pulse = 0.5f;
   bool pulseDirection = false;

   // Interface:
   MVBUTTON *point[POINTS];
   for (int c=0; c<POINTS; c++)
   {
         point[c] = new MVBUTTON();
         point[c]->setFont(font);
         point[c]->setSizes(16, 16);
         point[c]->setStaticStatus(false);
         point[c]->setLimitStatus(true);
         point[c]->setLimits(5, 44, MODULE_SIZEX-5, MODULE_SIZEY-95);
         char _pname[20];
         sprintf(_pname, "%d", c+1);
         point[c]->setCaption(_pname);
         point[c]->setColor(0.0f, 0.8f, 0.0f, 0.5f);
         point[c]->setPosition((float) ((c+1)*MODULE_SIZEX/(POINTS+1)), (float) (MODULE_SIZEY/2));
   }
   
   // Control points:
   MVVECTOR3 cPoint[POINTS]; 
   EXF_ZEROMEMORY(cPoint, sizeof(MVVECTOR3)*POINTS);   
  
  
   ////////////////
   // Module slide:
   MVTEXTURE *slideTexture = new MVTEXTURE();
   slideTexture->load("mixingparabola_slide.tga");
   MVWINDOW *slideWindow = new MVWINDOW();
   slideWindow->setFont(font);
   slideWindow->setCaption("Slide");
   slideWindow->setSizes(512, 440);
   slideWindow->setColor(1.0f, 1.0f, 1.0f, 1.0f);
   slideWindow->setTexture(slideTexture);
   slideWindow->setPosition(15, 85);
   
   MVVSCROLLBAR *slideScroll = new MVVSCROLLBAR();
   slideScroll->setMinValue(0.1f);
   slideScroll->setMaxValue(1.0f);
   slideScroll->setValue(1.0f);
   slideScroll->setPosition(495, 12);
   slideScroll->link(slideWindow);      
    
  
///////////////
// Mainloop: //
///////////////

   int mouseX = 0, mouseY = 0;
   bool mouseBL = false, mouseBR = false;
   bool done = false;
   while (!done)   
	{			
	      // Rendering:	   
	   	MVISIO::clear(true, true);	   		   	
			MVISIO::begin2D();			   
			   PASS_BACKGROUND;					     			  
			MVISIO::end2D();		     
			
			// Draw spline:			
			MVISIO::begin2D();		
			   
			   // Title:
			   font->putString(10, 13, MODULE_NAME);	  
			   glDisable(GL_TEXTURE_2D);
			   
			   // Draw curves:
            int d;
			   for (d=0; d<POINTS-2; d++) 
			   {
			         // Compute params:
			         MVVECTOR2 p0 = MVVECTOR2(point[d]->getPosition().x, point[d]->getPosition().y);
			         MVVECTOR2 p1 = MVVECTOR2(point[d+1]->getPosition().x, point[d+1]->getPosition().y);
			         MVVECTOR2 p2 = MVVECTOR2(point[d+2]->getPosition().x, point[d+2]->getPosition().y);	
			         p0 += MVVECTOR2((float) point[d]->getSizeX()/2.0f, (float) point[d]->getSizeY()/2.0f);		         
			         p1 += MVVECTOR2((float) point[d+1]->getSizeX()/2.0f, (float) point[d+1]->getSizeY()/2.0f);		         
			         p2 += MVVECTOR2((float) point[d+2]->getSizeX()/2.0f, (float) point[d+2]->getSizeY()/2.0f);		         
			               
			         float a, b, c;
			         parametricCurveBis(p0, p1, p2, &a, &b, &c);		
			         param[d*3]   = a;
			         param[d*3+1] = b;
			         param[d*3+2] = c;	  			        
			         
			         // Draw lines:
			         glLineWidth(LINE_WIDTH);	
			         glBegin(GL_LINE_STRIP);					         
			         for (int x=0; x<ACCURACY; x++)
			         {					               			                           
			               float _x = (float) x;			         
			               MVVECTOR2 p;
			               p.x = _x;
      			         p.y = pow(_x, 2.0f)*a + b*_x + c;
                  
			               // Compute and display curbe:		 			               
			               float color = (float) d;
                        if (d==0)
			                     glColor4f(1.0f, 0.5f, 0.2f, 0.7f);			      
                        else
                              glColor4f(0.2f, 1.0f, 0.2f, 0.7f);			      
			               glVertex2f(p.x, p.y);			           			               
			         }			               
			         glEnd();					   
			   }			   
			   
            /////////////          
			   {
			         // Compute params:
			         MVVECTOR2 p0 = MVVECTOR2(point[0]->getPosition().x, point[0]->getPosition().y);
			         MVVECTOR2 p1 = MVVECTOR2(point[1]->getPosition().x, point[1]->getPosition().y);
			         MVVECTOR2 p2 = MVVECTOR2(point[2]->getPosition().x, point[2]->getPosition().y);	
                  MVVECTOR2 p3 = MVVECTOR2(point[3]->getPosition().x, point[3]->getPosition().y);	
			         p0 += MVVECTOR2((float) point[0]->getSizeX()/2.0f, (float) point[0]->getSizeY()/2.0f);		         
			         p1 += MVVECTOR2((float) point[1]->getSizeX()/2.0f, (float) point[1]->getSizeY()/2.0f);		         
			         p2 += MVVECTOR2((float) point[2]->getSizeX()/2.0f, (float) point[2]->getSizeY()/2.0f);		         
                  p3 += MVVECTOR2((float) point[3]->getSizeX()/2.0f, (float) point[3]->getSizeY()/2.0f);		         
			               
			         float a1, b1, c1, a2, b2, c2;
			         parametricCurveBis(p0, p1, p2, &a1, &b1, &c1);		
                  parametricCurveBis(p1, p2, p3, &a2, &b2, &c2);			       		        
			         
			         // Draw lines:
			         glLineWidth(LINE_WIDTH);	
			         glBegin(GL_LINE_STRIP);					         			     
                  for (float t=0.0f; t<=1.0f; t=t+0.005f)
			         {		
                        float _x = p1.x;
                        float dist = absf(p2.x - p1.x);
                        _x = _x+dist*t;
                        float ratio = (_x-p1.x)/dist;
                        
                       // printf("Cycle: %f, ratio: %f, dist: %f, t:%f\n", _x, ratio, dist, t); 
			               MVVECTOR2 _p1, _p2;
                        
			               _p1.x = _x;
      			         _p1.y = pow(_x, 2.0f)*a1 + b1*_x + c1;
                        _p2.x = _x;
      			         _p2.y = pow(_x, 2.0f)*a2 + b2*_x + c2;
			               
                        // Compute and display curbe:		 			               			               
                        glColor4f(1.0f, 1.0f, pulse, pulse);			            
			               glVertex2f(t*_p2.x+(1.0f-t)*_p1.x, t*_p2.y+(1.0f-t)*_p1.y);			           			               
			         }			               
			         glEnd();					   
			   }			   
            /*
			   // Draw mixed parabole:
			   {
			         // Draw lines:
			         glLineWidth(LINE_WIDTH);	
			         glBegin(GL_LINE_STRIP);					   			 	   	   
			         for (int x=0; x<ACCURACY; x++)
			         {					               
			               MVVECTOR2 p0 = MVVECTOR2(point[1]->getPosition().x, point[1]->getPosition().y);
			               MVVECTOR2 p1 = MVVECTOR2(point[2]->getPosition().x, point[2]->getPosition().y);			                           
			               float _x = (float) x;			         
			               MVVECTOR2 p = mixedParable(_x, param[0], param[1], param[2], param[3], param[4], param[5], p0, p1); 
			             
			               // Compute and display curbe:		 			               
			               float color = (float) d;
			               glColor4f(1.0f, 1.0f, pulse, pulse);
			               glVertex2f(p.x, p.y);			           			               
			         }			               
			         glEnd();					         
			   }*/
			MVISIO::end2D();
			
			// Draw control points:
			MVISIO::begin2D();
			   PASS_QUITBUTTON;			   
			   for (int c=0; c<POINTS; c++)
			         point[c]->pass();		
			   slideWindow->setColor(1.0f, 1.0f, 1.0f, slideScroll->getValue());
			   slideWindow->pass();	  
			MVISIO::end2D();
			
			////////////////
			// Check events:			
			SDL_Event _event;			
			//SDL_PumpEvents();						
			while (SDL_PollEvent(&_event)) 
			{
					// Quit by click on X:
					if (_event.type == SDL_QUIT) 
							done = true;	
					
					// Resize?
				   if (_event.type == SDL_VIDEORESIZE)
				         MVSETTINGS::resize(0, 0, _event.resize.w, _event.resize.h);							
               
               // Quit by pressing ESC:
					if (_event.type == SDL_KEYDOWN) 
					{
							// Quit by ESC:
							if (_event.key.keysym.sym == SDLK_ESCAPE) 
							       done = true;															
			      }															
			}         
			
			// Update mouse:
			if (SDL_GetMouseState(&mouseX, &mouseY)&SDL_BUTTON(1))
					mouseBL = true;
			else 
					mouseBL = false;
			
			// Check MVisio events:
			MVID id=0;
			MVELEMENT::manageGUIEvents(0, mouseX, mouseY, mouseBL, mouseBR, &id);  
			if (PRESSED_QUITBUTTON(id))
			      done = true;			
			
			Sleep(10);
			MVISIO::swap();				

        	// Pulse:
			if (pulse >= 1.0f || pulse < 0.3f)
	            pulseDirection = !pulseDirection;
	      if (pulseDirection)
	            pulse += 0.04f;
	      else
	            pulse -= 0.04f;
	}      
   
   // Done:
   MVISIO::free();
   return 0;
}
